@@ -9,7 +9,7 @@ use crate::value::Value;
 
 /// Represents an open block of code that does not yet have updated
 /// jump targets.
-#[derive(Debug)]
+#[cfg_attr(feature = "internal_debug", derive(Debug))]
 enum PendingBlock {
     Branch(usize),
     Loop(usize),
@@ -17,7 +17,7 @@ enum PendingBlock {
 }
 
 /// Provides a convenient interface to creating instructions for the VM.
-#[derive(Debug)]
+#[cfg_attr(feature = "internal_debug", derive(Debug))]
 pub struct Compiler<'source> {
     instructions: Instructions<'source>,
     blocks: BTreeMap<&'source str, Instructions<'source>>,
@@ -25,17 +25,11 @@ pub struct Compiler<'source> {
     current_line: usize,
 }
 
-impl<'source> Default for Compiler<'source> {
-    fn default() -> Self {
-        Compiler::new("<unknown>")
-    }
-}
-
 impl<'source> Compiler<'source> {
     /// Creates an empty compiler
-    pub fn new(file: &'source str) -> Compiler<'source> {
+    pub fn new(file: &'source str, source: &'source str) -> Compiler<'source> {
         Compiler {
-            instructions: Instructions::new(file),
+            instructions: Instructions::new(file, source),
             blocks: BTreeMap::new(),
             pending_block: Vec::new(),
             current_line: 0,
@@ -245,7 +239,8 @@ impl<'source> Compiler<'source> {
             }
             ast::Stmt::Block(block) => {
                 self.set_location_from_span(block.span());
-                let mut sub_compiler = Compiler::new(self.instructions.name());
+                let mut sub_compiler =
+                    Compiler::new(self.instructions.name(), self.instructions.source());
                 sub_compiler.set_line(self.current_line);
                 for node in &block.body {
                     sub_compiler.compile_stmt(node)?;
