@@ -28,29 +28,55 @@ pub struct Template<'env> {
 
 impl<'env> fmt::Debug for Template<'env> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_struct("Template")
-            .field("name", &self.name)
-            .field("instructions", &self.compiled.instructions)
-            .field("blocks", &self.compiled.blocks)
-            .field("initial_auto_escape", &self.initial_auto_escape)
-            .finish()
+        let mut ds = f.debug_struct("Template");
+        ds.field("name", &self.name);
+        #[cfg(feature = "internal_debug")]
+        {
+            ds.field("instructions", &self.compiled.instructions);
+            ds.field("blocks", &self.compiled.blocks);
+        }
+        ds.field("initial_auto_escape", &self.initial_auto_escape);
+        ds.finish()
     }
 }
 
 /// Represents a compiled template in memory.
 #[derive(Clone)]
 pub(crate) struct CompiledTemplate<'source> {
-    source: &'source str,
     instructions: Instructions<'source>,
     blocks: BTreeMap<&'source str, Instructions<'source>>,
 }
 
 impl<'env> fmt::Debug for CompiledTemplate<'env> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_struct("CompiledTemplate")
-            .field("instructions", &self.instructions)
-            .field("blocks", &self.blocks)
-            .finish()
+        let mut ds = f.debug_struct("CompiledTemplate");
+        #[cfg(feature = "internal_debug")]
+        {
+            ds.field("instructions", &self.instructions);
+            ds.field("blocks", &self.blocks);
+        }
+        ds.finish()
+    }
+}
+
+fn attach_basic_debug_info<T>(rv: Result<T, Error>, source: &str) -> Result<T, Error> {
+    #[cfg(feature = "debug")]
+    {
+        match rv {
+            Ok(rv) => Ok(rv),
+            Err(mut err) => {
+                err.debug_info = Some(crate::error::DebugInfo {
+                    template_source: Some(source.to_string()),
+                    ..Default::default()
+                });
+                Err(err)
+            }
+        }
+    }
+    #[cfg(not(feature = "debug"))]
+    {
+        let _source = source;
+        rv
     }
 }
 
@@ -58,13 +84,19 @@ impl<'source> CompiledTemplate<'source> {
     pub(crate) fn from_name_and_source(
         name: &'source str,
         source: &'source str,
+    ) -> Result<CompiledTemplate<'source>, Error> {
+        attach_basic_debug_info(Self::_from_name_and_source_impl(name, source), source)
+    }
+
+    fn _from_name_and_source_impl(
+        name: &'source str,
+        source: &'source str,
     ) -> Result<CompiledTemplate<'source>, Error> {
         let ast = parse(source, name)?;
-        let mut compiler = Compiler::new(name);
+        let mut compiler = Compiler::new(name, source);
         compiler.compile_stmt(&ast)?;
         let (instructions, blocks) = compiler.finish();
         Ok(CompiledTemplate {
-            source,
             blocks,
             instructions,
         })
@@ -79,7 +111,7 @@ impl<'env> Template<'env> {
 
     /// Returns the source code of the template.
     pub fn source(&self) -> &str {
-        self.compiled.source
+        self.compiled.instructions.source()
     }
 
     /// Renders the template into a string.
@@ -154,16 +186,23 @@ impl<'source> fmt::Debug for Source<'source> {
 ///
 /// The environment holds references to the source the templates were created from.
 /// This makes it very inconvenient to pass around unless the templates are static
-/// strings.  For situations where you want to load dynamic templates and share the
-/// environment it's recommended to turn on the `source` feature and to use the
-/// [`Source`](crate::source::Source) type with the environment.
+/// strings.
+#[cfg_attr(
+    feature = "source",
+    doc = "
+For situations where you want to load dynamic templates and share the
+environment it's recommended to turn on the `source` feature and to use the
+[`Source`](crate::source::Source) type with the environment."
+)]
 #[derive(Clone)]
 pub struct Environment<'source> {
     templates: Source<'source>,
     filters: RcType<BTreeMap<&'source str, filters::BoxedFilter>>,
     tests: RcType<BTreeMap<&'source str, tests::BoxedTest>>,
-    globals: RcType<BTreeMap<&'source str, Value>>,
+    pub(crate) globals: RcType<BTreeMap<&'source str, Value>>,
     default_auto_escape: RcType<dyn Fn(&str) -> AutoEscape + Sync + Send>,
+    #[cfg(feature = "debug")]
+    debug: bool,
 }
 
 impl<'source> Default for Environment<'source> {
@@ -214,12 +253,19 @@ fn no_auto_escape(_: &str) -> AutoEscape {
 /// let rv = expr.eval(context!(number => 15)).unwrap();
 /// assert!(rv.is_true());
 /// ```
-#[derive(Debug)]
 pub struct Expression<'env, 'source> {
     env: &'env Environment<'source>,
     instructions: Instructions<'source>,
 }
 
+impl<'env, 'source> fmt::Debug for Expression<'env, 'source> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Expression")
+            .field("env", &self.env)
+            .finish()
+    }
+}
+
 impl<'env, 'source> Expression<'env, 'source> {
     /// Evaluates the expression with some context.
     ///
@@ -259,6 +305,8 @@ impl<'source> Environment<'source> {
             tests: RcType::new(tests::get_builtin_tests()),
             globals: RcType::new(functions::get_globals()),
             default_auto_escape: RcType::new(default_auto_escape),
+            #[cfg(feature = "debug")]
+            debug: false,
         }
     }
 
@@ -273,6 +321,8 @@ impl<'source> Environment<'source> {
             tests: RcType::default(),
             globals: RcType::default(),
             default_auto_escape: RcType::new(no_auto_escape),
+            #[cfg(feature = "debug")]
+            debug: false,
         }
     }
 
@@ -290,6 +340,30 @@ impl<'source> Environment<'source> {
         self.default_auto_escape = RcType::new(f);
     }
 
+    /// Enable or disable the debug mode.
+    ///
+    /// When the debug mode is enabled the engine will dump out some of the
+    /// execution state together with the source information of the executing
+    /// template when an error is created.  The cost of this is relatively
+    /// high as the data including the template source is cloned.
+    ///
+    /// However providing this information greatly improves the debug information
+    /// that the template error provides.  When debug is enabled errors will
+    /// return a [`DebugInfo`](crate::error::DebugInfo) object from
+    /// [`Error::debug_info`](crate::error::Error::debug_info).
+    ///
+    /// This requires the `debug` feature.
+    #[cfg(feature = "debug")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "debug")))]
+    pub fn set_debug(&mut self, enabled: bool) {
+        self.debug = enabled;
+    }
+
+    #[cfg(feature = "debug")]
+    pub(crate) fn debug(&self) -> bool {
+        self.debug
+    }
+
     /// Sets the template source for the environment.
     ///
     /// This helps when working with dynamically loaded templates.  For more
@@ -375,8 +449,15 @@ impl<'source> Environment<'source> {
     /// be used as a minimal scripting language.  For more information and an
     /// example see [`Expression`].
     pub fn compile_expression(&self, expr: &'source str) -> Result<Expression<'_, 'source>, Error> {
+        attach_basic_debug_info(self._compile_expression(expr), expr)
+    }
+
+    pub fn _compile_expression(
+        &self,
+        expr: &'source str,
+    ) -> Result<Expression<'_, 'source>, Error> {
         let ast = parse_expr(expr)?;
-        let mut compiler = Compiler::new("<expression>");
+        let mut compiler = Compiler::new("<expression>", expr);
         compiler.compile_expr(&ast)?;
         let (instructions, _) = compiler.finish();
         Ok(Expression {
