@@ -1,3 +1,4 @@
+#[cfg(feature = "internal_debug")]
 use std::fmt;
 
 use crate::value::Value;
@@ -163,6 +164,7 @@ pub enum Instruction<'source> {
     Nop,
 }
 
+#[cfg(feature = "internal_debug")]
 impl<'source> fmt::Debug for Instruction<'source> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match *self {
@@ -224,7 +226,7 @@ impl<'source> fmt::Debug for Instruction<'source> {
     }
 }
 
-#[derive(Debug, Copy, Clone)]
+#[derive(Copy, Clone)]
 struct Loc {
     first_instruction: u32,
     line: u32,
@@ -236,15 +238,17 @@ pub struct Instructions<'source> {
     pub(crate) instructions: Vec<Instruction<'source>>,
     locations: Vec<Loc>,
     name: &'source str,
+    source: &'source str,
 }
 
 impl<'source> Instructions<'source> {
-    // Creates a new instructions object.
-    pub fn new(name: &'source str) -> Instructions<'source> {
+    /// Creates a new instructions object.
+    pub fn new(name: &'source str, source: &'source str) -> Instructions<'source> {
         Instructions {
             instructions: Vec::new(),
             locations: Vec::new(),
             name,
+            source,
         }
     }
 
@@ -253,13 +257,18 @@ impl<'source> Instructions<'source> {
         self.name
     }
 
-    // Returns an instruction by index
+    /// Returns the source reference.
+    pub fn source(&self) -> &'source str {
+        self.source
+    }
+
+    /// Returns an instruction by index
     #[inline(always)]
     pub fn get(&self, idx: usize) -> Option<&Instruction<'source>> {
         self.instructions.get(idx)
     }
 
-    // Returns an instruction by index mutably
+    /// Returns an instruction by index mutably
     pub fn get_mut(&mut self, idx: usize) -> Option<&mut Instruction<'source>> {
         self.instructions.get_mut(idx)
     }
@@ -300,6 +309,27 @@ impl<'source> Instructions<'source> {
         Some(loc.line as usize)
     }
 
+    /// Returns a list of all names referenced in the current block backwards
+    /// from the given pc.
+    #[cfg(feature = "debug")]
+    pub fn get_referenced_names(&self, idx: usize) -> Vec<&'source str> {
+        let mut rv = Vec::new();
+        for instr in self.instructions[..=idx].iter().rev() {
+            match instr {
+                Instruction::Lookup(name) => rv.push(*name),
+                Instruction::StoreLocal(name) => rv.push(*name),
+                Instruction::CallFunction(name) => rv.push(*name),
+                Instruction::PushLoop(true) => {
+                    rv.push("loop");
+                    break;
+                }
+                Instruction::PushLoop(false) | Instruction::PushContext => break,
+                _ => {}
+            }
+        }
+        rv
+    }
+
     /// Returns the number of instructions
     pub fn len(&self) -> usize {
         self.instructions.len()
@@ -312,6 +342,7 @@ impl<'source> Instructions<'source> {
     }
 }
 
+#[cfg(feature = "internal_debug")]
 impl<'source> fmt::Debug for Instructions<'source> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         struct InstructionWrapper<'a>(usize, &'a Instruction<'a>, Option<usize>);
@@ -344,5 +375,5 @@ impl<'source> fmt::Debug for Instructions<'source> {
 #[test]
 #[cfg(target_pointer_width = "64")]
 fn test_sizes() {
-    assert_eq!(std::mem::size_of::<Instruction>(), 24);
+    assert_eq!(std::mem::size_of::<Instruction>(), 32);
 }
