@@ -65,6 +65,13 @@
 //! assert_eq!(result.is_true(), true);
 //! ```
 //!
+//! # Debugging
+//!
+//! To better understand what's going on when a template syntax or rendering error
+//! happens you should turn on the built-in debug support.  Once enabled the errors
+//! created by the engine will include helpful error messages with information about
+//! where the error happened.  For more information see [`set_debug`](crate::Environment::set_debug).
+//!
 //! # Learn more
 //!
 //! - [`syntax`]: documentation of the template engine syntax.
@@ -77,15 +84,24 @@
 //!
 //! # Optional Features
 //!
+//! MiniJinja comes with a lot of optional features, some of which are turned on by
+//! default.  If you plan on using MiniJinja in a library, please consider turning
+//! off all default features and to opt-in explicitly into the ones you actually
+//! need.
+//!
 //! There are some additional features that can be enabled:
 //!
-//! - `source`: enables the [`Source`] type which helps with dynamic loading of templates.
+//! - `source`: enables the `Source` type which helps with dynamic loading of templates.
 //! - `memchr`: enables the `memchr` dependency which provides performance improvements
 //!   for the parser.
 //! - `v_htmlescape`: enables the `v_htmlescape` dependency which implements a faster HTML
 //!   escaping algorithm.
 //! - `unstable_machinery`: provides access to the internal machinery of the engine.  This
 //!   is a forever unstable API which mainly exists to aid debugging complex issues.
+//! - `json`: When enabled the `tojson` filter is added as builtin filter.
+//! - `urlencode`: When enabled the `urlencode` filter is added as builtin filter.
+//! - `preserve_order`: When enable the internal value implementation uses an indexmap
+//!   which preserves the original order of maps and structs.
 //!
 //! Additionally to cut down on size of the engine some default
 //! functionality can be removed:
@@ -100,10 +116,14 @@
 //!   is disabled sending types across threads is often not possible.  Thread bounds
 //!   of things like callbacks however are not changing which means code that uses
 //!   MiniJinja still needs to be threadsafe.
-//! - `json`: When enabled the `tojson` filter is added as builtin filter.
-//! - `urlencode`: When enabled the `urlencode` filter is added as builtin filter.
+//! - `debug`: if this feature is removed some debug functionality of the engine is
+//!   removed as well.  This mainly affects the quality of error reporting.
+//! - `key_interning`: if this feature is removed the automatic string interning in
+//!   the value type is disabled.  The default behavior can cut down on the memory
+//!   consumption of the value type by interning all string keys used in values.
 #![allow(clippy::cognitive_complexity)]
 #![cfg_attr(docsrs, feature(doc_cfg))]
+#![doc(html_logo_url = "https://github.com/mitsuhiko/minijinja/raw/main/artwork/logo-square.png")]
 mod key;
 
 mod ast;
@@ -131,6 +151,9 @@ pub use self::environment::{Environment, Expression, Template};
 pub use self::error::{Error, ErrorKind};
 pub use self::utils::{AutoEscape, HtmlEscape};
 
+#[cfg(feature = "debug")]
+pub use self::error::DebugInfo;
+
 #[cfg(feature = "source")]
 pub use self::source::Source;
 
