@@ -1,5 +1,4 @@
 use std::collections::BTreeMap;
-use std::convert::TryFrom;
 use std::fmt::{self, Write};
 use std::sync::atomic::{AtomicUsize, Ordering};
 
@@ -81,7 +80,7 @@ impl fmt::Display for LoopState {
     }
 }
 
-#[derive(Debug)]
+#[cfg_attr(feature = "internal_debug", derive(Debug))]
 pub struct Loop<'env> {
     locals: BTreeMap<&'env str, Value>,
     with_loop_var: bool,
@@ -100,6 +99,7 @@ pub enum Frame<'env, 'vm> {
     Loop(Loop<'env>),
 }
 
+#[cfg(feature = "internal_debug")]
 impl<'env, 'vm> fmt::Debug for Frame<'env, 'vm> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
@@ -116,7 +116,7 @@ impl<'env, 'vm> fmt::Debug for Frame<'env, 'vm> {
     }
 }
 
-#[derive(Debug, Default)]
+#[cfg_attr(feature = "internal_debug", derive(Debug))]
 pub struct Stack {
     values: Vec<Value>,
 }
@@ -139,12 +139,54 @@ impl Stack {
     }
 }
 
-#[derive(Default, Debug)]
+#[derive(Default)]
+#[cfg_attr(feature = "internal_debug", derive(Debug))]
 pub struct Context<'env, 'vm> {
     stack: Vec<Frame<'env, 'vm>>,
 }
 
 impl<'env, 'vm> Context<'env, 'vm> {
+    /// Freezes the context.
+    ///
+    /// This implementation is not particularly beautiful and highly inefficient.
+    /// Since it's only used for the debug support changing this is not too
+    /// critical.
+    #[cfg(feature = "debug")]
+    fn freeze<'a>(&'a self, env: &'a Environment) -> BTreeMap<&'a str, Value> {
+        let mut rv = BTreeMap::new();
+
+        for ctx in self.stack.iter() {
+            let (lookup_base, cont) = match ctx {
+                // if we hit a chain frame we dispatch there and never
+                // recurse
+                Frame::Chained { base } => return base.freeze(env),
+                Frame::Isolate { value } => (value, false),
+                Frame::Merge { value } => (value, true),
+                Frame::Loop(Loop {
+                    locals,
+                    controller,
+                    with_loop_var,
+                    ..
+                }) => {
+                    rv.extend(locals.iter().map(|(k, v)| (*k, v.clone())));
+                    if *with_loop_var {
+                        rv.insert("loop", Value::from_rc_object(controller.clone()));
+                    }
+                    continue;
+                }
+            };
+
+            if !cont {
+                rv.clear();
+                rv.extend(env.globals.iter().map(|(k, v)| (*k, v.clone())));
+            }
+
+            rv.extend(lookup_base.iter_as_str_map());
+        }
+
+        rv
+    }
+
     /// Stores a variable in the context.
     pub fn store(&mut self, key: &'env str, value: Value) {
         self.current_loop().locals.insert(key, value);
@@ -232,13 +274,16 @@ pub struct State<'vm, 'env> {
 
 impl<'vm, 'env> fmt::Debug for State<'vm, 'env> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_struct("State")
-            .field("ctx", &self.ctx)
-            .field("name", &self.name)
-            .field("current_block", &self.current_block)
-            .field("auto_escape", &self.auto_escape)
-            .field("env", &self.env)
-            .finish()
+        let mut ds = f.debug_struct("State");
+        #[cfg(feature = "internal_debug")]
+        {
+            ds.field("ctx", &self.ctx);
+        }
+        ds.field("name", &self.name);
+        ds.field("current_block", &self.current_block);
+        ds.field("auto_escape", &self.auto_escape);
+        ds.field("env", &self.env);
+        ds.finish()
     }
 }
 
@@ -299,10 +344,24 @@ impl<'vm, 'env> State<'vm, 'env> {
             ))
         }
     }
+
+    #[cfg(feature = "debug")]
+    fn make_debug_info(
+        &self,
+        pc: usize,
+        instructions: &Instructions<'_>,
+    ) -> crate::error::DebugInfo {
+        let referenced_names = instructions.get_referenced_names(pc);
+        crate::error::DebugInfo {
+            template_source: Some(instructions.source().to_string()),
+            context: Some(Value::from(self.ctx.freeze(self.env))),
+            referenced_names: Some(referenced_names.iter().map(|x| x.to_string()).collect()),
+        }
+    }
 }
 
 /// Helps to evaluate something.
-#[derive(Debug)]
+#[cfg_attr(feature = "internal_debug", derive(Debug))]
 pub struct Vm<'env> {
     env: &'env Environment<'env>,
 }
@@ -347,7 +406,7 @@ impl<'env> Vm<'env> {
         output: &mut String,
     ) -> Result<Option<Value>, Error> {
         let initial_auto_escape = state.auto_escape;
-        let mut stack = Stack::default();
+        let mut stack = Stack { values: Vec::new() };
         let mut auto_escape_stack = vec![];
         let mut capture_stack = vec![];
         let mut block_stack = vec![];
@@ -359,6 +418,12 @@ impl<'env> Vm<'env> {
                 if let Some(lineno) = instructions.get_line(pc) {
                     err.set_location(instructions.name(), lineno);
                 }
+                #[cfg(feature = "debug")]
+                {
+                    if self.env.debug() && err.debug_info.is_none() {
+                        err.debug_info = Some(state.make_debug_info(pc, &instructions));
+                    }
+                }
                 return Err(err);
             }};
         }
@@ -465,7 +530,7 @@ impl<'env> Vm<'env> {
                     let mut map = BTreeMap::new();
                     for _ in 0..*pair_count {
                         let value = stack.pop();
-                        let key: Key = try_ctx!(TryFrom::try_from(stack.pop()));
+                        let key: Key = try_ctx!(stack.pop().try_into_key());
                         map.insert(key, value);
                     }
                     stack.push(Value::from(map));
