@@ -4,7 +4,7 @@ use minijinja::value::Value;
 
 #[test]
 fn test_for_loop() {
-    let mut c = Compiler::default();
+    let mut c = Compiler::new("<unknown>", "");
     c.add(Instruction::Lookup("items"));
     c.start_for_loop(true);
     c.add(Instruction::Emit);
@@ -16,7 +16,7 @@ fn test_for_loop() {
 
 #[test]
 fn test_if_branches() {
-    let mut c = Compiler::default();
+    let mut c = Compiler::new("<unknown>", "");
     c.add(Instruction::Lookup("false"));
     c.start_if();
     c.add(Instruction::EmitRaw("nope1"));
@@ -34,7 +34,7 @@ fn test_if_branches() {
 
 #[test]
 fn test_bool_ops() {
-    let mut c = Compiler::default();
+    let mut c = Compiler::new("<unknown>", "");
 
     c.start_sc_bool();
     c.add(Instruction::Lookup("first"));
@@ -49,7 +49,7 @@ fn test_bool_ops() {
 
 #[test]
 fn test_const() {
-    let mut c = Compiler::default();
+    let mut c = Compiler::new("<unknown>", "");
 
     c.add(Instruction::LoadConst(Value::from("a")));
     c.add(Instruction::LoadConst(Value::from(42)));
